# Copyright 2015 Planet Labs, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import json
import re


def _get_contents_as_string(obj):
    return obj.get()['Body'].read()


def test_push_file(archive, random_metadata, tmpfile, s3_object):
    expected_content = 'mwahaha'.encode('utf-8')
    f = tmpfile(expected_content)
    url = archive.prepare_metadata_and_push(f, **random_metadata)
    from_s3 = s3_object(url)
    assert _get_contents_as_string(from_s3) == expected_content
    metadata = from_s3.get()['Metadata'].get('datalake')
    assert metadata is not None
    metadata = json.loads(metadata)
    common_keys = set(metadata.keys()).intersection(random_metadata.keys())
    assert common_keys == set(random_metadata.keys())


def test_push_large_file(
        monkeypatch, archive, random_metadata, tmpfile, s3_object):
    monkeypatch.setenv('DATALAKE_CHUNK_SIZE_MB', '5')
    expected_content = ('big data' * 1024 * 1024).encode('utf-8')
    f = tmpfile(expected_content)
    url = archive.prepare_metadata_and_push(f, **random_metadata)
    from_s3 = s3_object(url)
    assert _get_contents_as_string(from_s3) == expected_content
    metadata = from_s3.get()['Metadata'].get('datalake')
    assert metadata is not None
    metadata = json.loads(metadata)
    common_keys = set(metadata.keys()).intersection(random_metadata.keys())
    assert common_keys == set(random_metadata.keys())
    for k in common_keys:
        assert metadata[k] == random_metadata[k]


def test_file_url(archive, random_metadata, tmpfile):
    expected_content = 'mwahaha'
    f = tmpfile(expected_content)
    url = archive.prepare_metadata_and_push(f, **random_metadata)
    assert bool(re.match(r'^s3://datalake-test/[a-z0-9]{40}/data$', url))
